//
//  MagnetometerController.m
//  Magnetometer
//
//  Created by Phidgets on 2015-07-27.
//  Copyright (c) 2015 Phidgets. All rights reserved.
//

#import "MagnetometerController.h"

@implementation MagnetometerController

- (void)windowWillClose:(NSNotification*)aNotification{
    if(ch){
        //Safely close the example
        Phidget_setOnAttachHandler((PhidgetHandle)ch, NULL, NULL);
        Phidget_setOnDetachHandler((PhidgetHandle)ch, NULL, NULL);
        Phidget_setOnErrorHandler((PhidgetHandle)ch, NULL, NULL);
        Phidget_close((PhidgetHandle)ch);
        PhidgetMagnetometer_delete(&ch);
        ch = NULL;
    }
}

-(void)awakeFromNib{
    PhidgetReturnCode result;
    
    //Manage GUI
    NSRect frame = [mainWindow frame];
    frame.size.height = [mainWindow minSize].height;
    [mainWindow setFrame:frame display:NO];
    [mainWindow center];
    [mainWindow setDelegate:self];
    [phidgetInfoBoxView fillPhidgetInfo:nil];
    [phidgetInfoBox setHidden:NO];
    
    result = PhidgetMagnetometer_create(&ch);
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
    
    result = [self initChannel:(PhidgetHandle)ch];
    if(result != EPHIDGET_OK){
        [self addError:result];
    }

    result = PhidgetMagnetometer_setOnMagneticFieldChangeHandler(ch, gotMagneticFieldChangeData, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        [self addError:result];
    }
    
    [PhidgetInfoBox openCmdLine:(PhidgetHandle)ch];
}

-(PhidgetReturnCode)initChannel:(PhidgetHandle) channel{
    PhidgetReturnCode result;
    
    result = Phidget_setOnAttachHandler(channel, gotAttach, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        return result;
    }
    
    result = Phidget_setOnDetachHandler(channel, gotDetach, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        return result;
    }
    
    result = Phidget_setOnErrorHandler(channel, gotError, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        return result;
    }
    
    /*
     * Please review the Phidget22 channel matching documentation for details on the device
     * and class architecture of Phidget22, and how channels are matched to device features.
     */
    
    /*
     * Specifies the serial number of the device to attach to.
     * For VINT devices, this is the hub serial number.
     *
     * The default is any device.
     */
    // Phidget_setDeviceSerialNumber(ch, <YOUR DEVICE SERIAL NUMBER>);
    
    /*
     * For VINT devices, this specifies the port the VINT device must be plugged into.
     *
     * The default is any port.
     */
    // Phidget_setHubPort(ch, 0);
    
    /*
     * Specifies that the channel should only match a VINT hub port.
     * The only valid channel id is 0.
     *
     * The default is 0 (false), meaning VINT hub ports will never match
     */
    // Phidget_setIsHubPortDevice(ch, 1);
    
    /*
     * Specifies which channel to attach to.  It is important that the channel of
     * the device is the same class as the channel that is being opened.
     *
     * The default is any channel.
     */
    // Phidget_setChannel(ch, 0);
    
    /*
     * In order to attach to a network Phidget, the program must connect to a Phidget22 Network Server.
     * In a normal environment this can be done automatically by enabling server discovery, which
     * will cause the client to discovery and connect to available servers.
     *
     * To force the channel to only match a network Phidget, set remote to 1.
     */
    // PhidgetNet_enableServerDiscovery(PHIDGETSERVER_DEVICE);
    // Phidget_setIsRemote(ch, 1);
    
    return EPHIDGET_OK;
}

#pragma mark Event callbacks
static void gotAttach(PhidgetHandle phid, void *context){
    [(__bridge id)context performSelectorOnMainThread:@selector(onAttachHandler)
                                           withObject:nil
                                        waitUntilDone:NO];
}
static void gotDetach(PhidgetHandle phid, void *context){
    [(__bridge id)context performSelectorOnMainThread:@selector(onDetachHandler)
                                           withObject:nil
                                        waitUntilDone:NO];
}
static void gotError(PhidgetHandle phid, void *context, Phidget_ErrorEventCode errcode, const char *error) {
    [(__bridge id)context performSelectorOnMainThread:@selector(errorHandler:)
                                           withObject:[NSArray arrayWithObjects:[NSNumber numberWithInt:errcode], [NSString stringWithUTF8String:error], nil]
                                        waitUntilDone:NO];
}
void gotMagneticFieldChangeData(PhidgetMagnetometerHandle phid, void *context, const double fieldStrength[3], double timestamp){
    [(__bridge id)context performSelectorOnMainThread:@selector(onMagneticFieldChangeHandler:)
                                           withObject:[NSArray arrayWithObjects: [NSNumber numberWithDouble:fieldStrength[0]],
                                                       [NSNumber numberWithDouble:fieldStrength[1]],[NSNumber numberWithDouble:fieldStrength[2]],[NSNumber numberWithDouble:timestamp],nil]
                                        waitUntilDone:NO];
}

#pragma mark Attach, detach, data, and error events
- (void)onAttachHandler{
    uint32 minDataInterval,maxDataInterval, dataInterval;
    double minMagneticFieldChangeTrigger, maxMagneticFieldChangeTrigger, magneticFieldChangeTrigger;
    
    //Get information from channel that will allow us to configure the GUI properly
    PhidgetMagnetometer_getMaxDataInterval(ch, &maxDataInterval);
    PhidgetMagnetometer_getMinDataInterval(ch, &minDataInterval);
    PhidgetMagnetometer_getDataInterval(ch, &dataInterval);
    PhidgetMagnetometer_getMinMagneticFieldChangeTrigger(ch, &minMagneticFieldChangeTrigger);
    PhidgetMagnetometer_getMaxMagneticFieldChangeTrigger(ch, &maxMagneticFieldChangeTrigger);
    PhidgetMagnetometer_getMagneticFieldChangeTrigger(ch, &magneticFieldChangeTrigger);
    
    //Adjust GUI based on information from channel
    [phidgetInfoBoxView fillPhidgetInfo:(PhidgetHandle)ch];
    [settingsBox setHidden:NO];
    [dataBox setHidden:NO];
    [magneticFieldVectorBox setHidden:NO];
    
    
    //Configure the data interval slider with information read from channel
    [minDataIntervalLabel setStringValue:[NSString stringWithFormat:@"%d ms",minDataInterval]];
    [maxDataIntervalLabel setStringValue:[NSString stringWithFormat:@"%d ms",maxDataInterval]];
    [dataIntervalText setStringValue:[NSString stringWithFormat:@"%d ms", dataInterval]];
    [dataIntervalSlider setMinValue:(double)minDataInterval];
    [dataIntervalSlider setMaxValue:(double)maxDataInterval];
    [dataIntervalSlider setIntValue:dataInterval];
    
    //Configure change trigger slider with information read from channel
    [maxChangeTriggerLabel setStringValue:[NSString stringWithFormat:@"%0.3f G",maxMagneticFieldChangeTrigger]];
    [minChangeTriggerLabel setStringValue:[NSString stringWithFormat:@"%0.3f G",minMagneticFieldChangeTrigger]];
    [changeTriggerText setStringValue:[NSString stringWithFormat:@"%0.3f G", magneticFieldChangeTrigger]];
    [changeTriggerSlider setMaxValue:maxMagneticFieldChangeTrigger];
    [changeTriggerSlider setMinValue:minMagneticFieldChangeTrigger];
    [changeTriggerSlider setIntValue:magneticFieldChangeTrigger];
    
    //for graphics
    [magFieldView setCircleRadius:0.5];
    
    //Adjusting window height
    NSRect frame = [mainWindow frame];
    frame.size.height = [mainWindow maxSize].height;
    [mainWindow setFrame:frame display:NO];
    [mainWindow center];
}

- (void)onDetachHandler{
    //Reset
    [settingsBox setHidden:YES];
    [dataBox setHidden:YES];
    [magneticFieldVectorBox setHidden:YES];
    [phidgetInfoBoxView fillPhidgetInfo:nil];
    [calibrationStatusText setStringValue:@""];
    
    //Adjusting window height
    NSRect frame = [mainWindow frame];
    frame.size.height = [mainWindow minSize].height;
    [mainWindow setFrame:frame display:NO];
    [mainWindow center];
}

static int errorCounter = 0;
-(void) outputError:(const char *)errorString{
    errorCounter++;
    NSAttributedString *outputString = [[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"%@\n",[NSString stringWithUTF8String:errorString]]];
    [[errorEventLog textStorage] beginEditing];
    [[errorEventLog textStorage] appendAttributedString:outputString];
    [[errorEventLog textStorage] endEditing];
    
    [errorEventLogCounter setIntValue:errorCounter];
    if(![errorEventLogWindow isVisible])
        [errorEventLogWindow setIsVisible:YES];
}

-(void)errorHandler:(NSArray *)errorEventData{
    const char* errorString = [[errorEventData objectAtIndex:1] UTF8String];
    [self outputError:errorString];
}

-(void)addError:(PhidgetReturnCode)result{
    const char *errorString;
    Phidget_getErrorDescription(result, &errorString);
    [self outputError:errorString];
}

- (IBAction)clearErrorLog:(id)sender{
    [[errorEventLog textStorage] setAttributedString:[[NSAttributedString alloc] initWithString:@""]];
    [errorEventLogCounter setIntValue:0];
    errorCounter = 0;
}

- (void)onMagneticFieldChangeHandler:(NSArray *)magData{
    [xAxisText setStringValue:[NSString stringWithFormat:@"%.3f G",[[magData objectAtIndex:0]doubleValue]]];
    [yAxisText setStringValue:[NSString stringWithFormat:@"%.3f G",[[magData objectAtIndex:1]doubleValue]]];
    [zAxisText setStringValue:[NSString stringWithFormat:@"%.3f G",[[magData objectAtIndex:2]doubleValue]]];
    [timeStampText setStringValue:[NSString stringWithFormat:@"%.0f ms",[[magData objectAtIndex:3]doubleValue]]];
    [magFieldView setVector:[[magData objectAtIndex:0]doubleValue] :[[magData objectAtIndex:1]doubleValue] :[[magData objectAtIndex:2]doubleValue]];
}


#pragma mark GUI controls
- (IBAction)openParameterWindow:(id)sender{
    if(![parametersWindow isVisible]){
        [parametersWindow setIsVisible:YES];
        NSArray *compassParamsArray = [[NSUserDefaults standardUserDefaults] objectForKey:@"compassParams"];
        if(compassParamsArray == nil){
            [parametersText setStringValue:@"0.41595, 0.02233, 0.15236, -0.22662, 2.36152, 2.27444, 2.57652, -0.03768, -0.01708, -0.03607, -0.00707, -0.01822, -0.00787"];
        }
        else{
            [parametersText setStringValue:(NSString *)[compassParamsArray objectAtIndex:0]];
        }
    }
}

- (IBAction)closeParameterWindow:(id)sender{
    [parametersWindow setIsVisible:NO];
}

- (IBAction)setParameters:(id)sender{
    int returnValue = 0;
    NSArray *compassParamsArray;
    NSAlert *alert;
    NSString *errorString = @"Error!\nCalibration error occurred, ensure values are within proper range.";
    NSArray *strArray = [parametersText.stringValue componentsSeparatedByString:@","];
    if([[strArray objectAtIndex:0]  isEqual: @""]){
        [[NSUserDefaults standardUserDefaults] removeObjectForKey:@"compassParams"];
        [[NSUserDefaults standardUserDefaults] synchronize];
        parametersText.stringValue = @"0.41595, 0.02233, 0.15236, -0.22662, 2.36152, 2.27444, 2.57652, -0.03768, -0.01708, -0.03607, -0.00707, -0.01822, -0.00787";
        strArray = [parametersText.stringValue componentsSeparatedByString:@","];
        alert = [[NSAlert alloc] init];
        [alert setMessageText:@"Calibration parameters reset"];
        [alert runModal];
    }
    else if([strArray count] != 13){
        errorString = @"Error!\nInvalid number of parameters entered. Ensure there are 13 values";
        goto calibrationError;
    }
    
    //save params 
    compassParamsArray = [[NSArray alloc] initWithObjects:parametersText.stringValue, nil];
    [[NSUserDefaults standardUserDefaults] setObject:compassParamsArray forKey:@"compassParams"];
    [[NSUserDefaults standardUserDefaults] synchronize];
    
    returnValue = PhidgetMagnetometer_setCorrectionParameters(ch, [[strArray objectAtIndex:0]doubleValue],
                                                [[strArray objectAtIndex:1]doubleValue],
                                                [[strArray objectAtIndex:2]doubleValue],
                                                [[strArray objectAtIndex:3]doubleValue],
                                                [[strArray objectAtIndex:4]doubleValue],
                                                [[strArray objectAtIndex:5]doubleValue],
                                                [[strArray objectAtIndex:6]doubleValue],
                                                [[strArray objectAtIndex:7]doubleValue],
                                                [[strArray objectAtIndex:8]doubleValue],
                                                [[strArray objectAtIndex:9]doubleValue],
                                                [[strArray objectAtIndex:10]doubleValue],
                                                [[strArray objectAtIndex:11]doubleValue],
                                                [[strArray objectAtIndex:12]doubleValue]);
    if(returnValue == 0){
        [magFieldView setCircleRadius:[[strArray objectAtIndex:0]doubleValue]];
        [calibrationStatusText setTextColor:[NSColor colorWithRed:0.2 green:0.8 blue:0.2 alpha:1.0]];
        [calibrationStatusText setStringValue:@"Calibration Complete"];
    }
    if(returnValue != 0){
    calibrationError:
        alert = [[NSAlert alloc] init];
        [alert setMessageText:errorString];
        [alert runModal];
        [calibrationStatusText setTextColor:[NSColor redColor]];
        [calibrationStatusText setStringValue:@"Calibration Error"];
    }
    [parametersWindow setIsVisible:NO];
}

- (IBAction)setChangeTrigger:(id)sender{
    PhidgetReturnCode result;
    result = PhidgetMagnetometer_setMagneticFieldChangeTrigger(ch, [changeTriggerSlider doubleValue]);
    if(result != EPHIDGET_OK){
        [self addError:result];
        return;
    }
    [changeTriggerText setStringValue:[NSString stringWithFormat:@"%0.3f G", [changeTriggerSlider doubleValue]]];
}

- (IBAction)setDataInterval:(id)sender{
    PhidgetReturnCode result;
    result = PhidgetMagnetometer_setDataInterval(ch, [dataIntervalSlider intValue]);
    if(result != EPHIDGET_OK){
        [self addError:result];
        return;
    }
    [dataIntervalText setStringValue:[NSString stringWithFormat:@"%d ms", [dataIntervalSlider intValue]]];
}
@end
